------------------------------------------------------------------------
--  DlabDio5.vhd -- Digilab DIO5 Complex CPLD Configuration
------------------------------------------------------------------------
--  Author:  Gene Apperson
--           Copyright 2002 Digilent, Inc.
------------------------------------------------------------------------
--  This module contains one of the standard configurations for the CPLD on
--  the Digilab DIO5 board. This CPLD implements the more complex bus
--  oriented interface used by a system board to interract with the DIO5 
--  board. 
--
--  The following on-board i/o devices are accessed via this CPLD logic:
--      16 push buttons
--      8 switches
--      4 digit, seven segment LED display
--      16 discrete led's
--
--  The sixteen buttons on the DIO5 are arranged as a decimal keypad and
--  six directly connected function buttons. All buttons are routed through
--  the CPLD and are readable via the bus interface. Additionally, Buttons 0-9 
--  are encoded to BCD by the CPLD logic. The remaining six buttons pass
--  through the CPLD and connect directly ot the system board interface 
--  connectors. All buttons are debounced via two latches driven by the 
--  input clock.
--
--  The eight slide switches both bypass the CPLD and connect directly
--  to the interface connectors, as well as going into the CPLD to allow
--  reading via the bus interface.
--
--  The LCD display signals pass through the CPLD to provide logic level
--  conversion from the 5V logic levels of the LCD display to the 3.3V
--  logic levels used by system boards.
--
--  The VGA signals on the DIO5 board go directly to the connectors and are 
--  not connected to the CPLD.
--
--  The keyboard signals pass through the CPLD to provide logic level
--  shifting. The standard implementation provies an input only keyboard
--  and does not support writing to the keyboard.
--
------------------------------------------------------------------------
-- Revision History:
--  03/20/2003(GeneA): created
--  05/02/2003(GeneA): added pass through for keyboard clock and data
--  08/06/2003(ClintC): rearranged memory map to be as follows
--	address	 read	write
--	  000	 BtnLo	LedLo
--	  001	 BtnHi	LedHi
--	  010	 Swt		SsegLo
--	  010       -		SsegHi
--  08/11/2004(BarronB): Fixed LCLK mapping problem
------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity DlabDio5 is
    Port ( db       : inout std_logic_vector(7 downto 0);
           adr      : in std_logic_vector(5 downto 0);
           clk      : in std_logic;
           cs       : in std_logic;
           we       : in std_logic;
           oe       : in std_logic;
           lcen     : in std_logic;
           rgbLcd   : inout std_logic_vector(7 downto 0);
           lcden    : out std_logic;
           lcdrw    : out std_logic;
           lcdrs    : out std_logic;
           rgbBtnIn : in std_logic_vector(15 downto 0);
           rgbSwt   : in std_logic_vector(7 downto 0);
           encBtn   : out std_logic_vector(3 downto 0);
           rgbBtnOt : out std_logic_vector(5 downto 0);
           rgbLed   : out std_logic_vector(15 downto 0);
           rgbSsgAn : out std_logic_vector(3 downto 0);
           rgbSsgCa : out std_logic_vector(7 downto 0);
           kclk     : out std_logic;
           kdat     : out std_logic;
           kcin     : in  std_logic;
           kdin     : in  std_logic);
end DlabDio5;

architecture Behavioral of DlabDio5 is

    ------------------------------------------------------------------------
    -- Component Declarations
    ------------------------------------------------------------------------


    ------------------------------------------------------------------------
    -- Constant and Signal Declarations
    ------------------------------------------------------------------------

    -- Internal registers for output devices
    signal  regLed     : std_logic_vector(15 downto 0);
    signal  regSsg     : std_logic_vector(15 downto 0);
    signal  regBtn     : std_logic_vector(15 downto 0);
    signal  rgbSyncBtn : std_logic_vector(15 downto 0);

    -- Other needed signals
    signal  busDout    : std_logic_vector(7 downto 0);
    signal  cntDig     : std_logic_vector(1 downto 0) := "00";
    signal  ssg        : std_logic_vector(6 downto 0);
    signal  dig        : std_logic_vector(3 downto 0);
    signal  an         : std_logic_vector(3 downto 0);

----------------------------------------------------------------------------
-- Module Implementation
----------------------------------------------------------------------------
    
begin

    ------------------------------------------------------------------------
    -- Bus drive control logic. Data bus is driven only when oe and cs are valid.
    ------------------------------------------------------------------------

    db <= busDout when (oe = '0' and cs = '0') else "ZZZZZZZZ";

    -- Output mux to place selected data on bus
    busDout <= rgbSyncBtn(7 downto 0) when adr = "000000" else
               rgbSyncBtn(15 downto 8) when adr = "000001" else
               rgbSwt when adr = "000010" else
               rgbLcd;

    ------------------------------------------------------------------------
    -- Registers for On-Board Output Devices	(LEDs and Sseg display)
    ------------------------------------------------------------------------

    -- The WE signal is driven from a global clock buffer.
    process (we)
        begin
            if (we'event and we = '1') then
                if (adr = "000000" and cs = '0') then regLed( 7 downto 0) <= db; end if;
                if (adr = "000001" and cs = '0') then regLed(15 downto 8) <= db; end if;
                if (adr = "000010" and cs = '0') then regSsg( 7 downto 0) <= db; end if;
                if (adr = "000011" and cs = '0') then regSsg(15 downto 8) <= db; end if;
            end if;
        end process;

    -- Assign outputs of the LED holding registers to the led outputs
    rgbLed  <= not regLed;

    ------------------------------------------------------------------------
    -- Synthesize control signals for the LCD display
    ------------------------------------------------------------------------

    rgbLcd <= db when (cs = '0' and we = '0') else "ZZZZZZZZ";
    lcden  <= '1' when (adr = "0001--" and cs = '0' and not ((we = '1') and (oe = '1'))) or lcen = '1' else '0';
    lcdrw  <= we;
    lcdrs  <= adr(0);

    ------------------------------------------------------------------------
    -- Get button inupts: All 16 button inputs are registered. Six buttons
    -- (A through F) are passed directly through to output pins on the CPLD,
    -- and ten buttons (0 through 9) are encoded to BCD.
    ------------------------------------------------------------------------

    -- Register all buttons
    process(rgbBtnIn, clk)
        begin
            if clk = '1' and clk'Event
		      then regBtn <= rgbBtnIn;
            end if;
        end process;
    rgbSyncBtn <= regBtn;

    -- Assign debounced pass-through buttons to their outputs
    rgbBtnOt <= rgbSyncBtn(15 downto 10);

    -- 10-4 Line BCD Encoder for Buttons
    encBtn <=   "0000" when rgbSyncBtn(0) = '1' else
                "0001" when rgbSyncBtn(1) = '1' else
                "0010" when rgbSyncBtn(2) = '1' else
                "0011" when rgbSyncBtn(3) = '1' else
                "0100" when rgbSyncBtn(4) = '1' else
                "0101" when rgbSyncBtn(5) = '1' else
                "0110" when rgbSyncBtn(6) = '1' else
                "0111" when rgbSyncBtn(7) = '1' else
                "1000" when rgbSyncBtn(8) = '1' else
                "1001" when rgbSyncBtn(9) = '1' else
                "1111";

    ------------------------------------------------------------------------
    --      Seven Segment Display Driver
    ------------------------------------------------------------------------

    -- Counter to cycle around the four digit numbers.
    process (clk)
        begin
            if clk = '1' and clk'Event then
                cntDig <= cntDig + 1;
            end if;
        end process;

    -- Seven Segment Display Decoder. This logic defines a four bit binary to
    -- seven segment decoder. The output produces a logic 1 for each segment
    -- that should be on and a logic 0 for each segment that should be off.
    -- Segment A is the least significant bit.
    ssg <= "0111111" when dig = "0000" else
  		 "0000110" when dig = "0001" else
 		 "1011011" when dig = "0010" else
 		 "1001111" when dig = "0011" else
 		 "1100110" when dig = "0100" else
 		 "1101101" when dig = "0101" else
 		 "1111101" when dig = "0110" else
 		 "0000111" when dig = "0111" else
 		 "1111111" when dig = "1000" else
 		 "1101111" when dig = "1001" else
           "1110111" when dig = "1010" else
           "1111100" when dig = "1011" else
           "0111001" when dig = "1100" else
           "1011110" when dig = "1101" else
           "1111001" when dig = "1110" else
           "1110001" when dig = "1111" else
		 "0000000";

    -- Digit multiplexer. This will enable each digit in succession
    -- and use a mux to select the appropriate data onto the cathode
    -- lines while driving the appropriate anode line to enable the 
    -- digit.
    an <= "1110" when cntDig = "00" else
          "1101" when cntDig = "01" else
          "1011" when cntDig = "10" else
          "0111" when cntDig = "11" else
          "1111";

    dig <= regSsg(15 downto 12) when cntDig = "00" else
           regSsg(11 downto  8) when cntDig = "01" else
           regSsg( 7 downto  4) when cntDig = "10" else
           regSsg( 3 downto  0);

    -- Map the internal signals to the pins on the CPLD.
    rgbSsgAn <= an;
    rgbSsgCa <= '1' & (not ssg);

    kclk <= kcin;
    kdat <= kdin;

end Behavioral;
