------------------------------------------------------------------------
--     DlabDio5.vhd -- Digilab DIO5 Simple CPLD Configuration
------------------------------------------------------------------------
-- Author:  Gene Apperson
--          Copyright 2002 Digilent, Inc.
------------------------------------------------------------------------
-- This module contains the standard configuration logic for the CPLD on
-- the Digilab DIO5 board. This CPLD implements the interface used by
-- the system board to interract with the DIO5 board. The following
-- on-board i/o devices are accessed via this CPLD logic:
--      10 push buttons
--      4 digit, seven segment LED display
--      16 discrete led's
--
--  The sixteen buttons on the DIO5 are arranged as a decimal keypad and
--  six directly connected function buttons. All buttons are routed through
--  the CPLD and are readable via the bus interface. Additionally, Buttons 0-9 
--  are encoded to BCD by the CPLD logic. The remaining six buttons also
--  bypass the CPLD and connect directly ot the system board
--  interface connectors.
--
--  The eight slide switches bypass the CPLD and connect directly
--  to the interface connectors.
--
--  The VGA port and PS/2 keyboard signals on the DIO5 board go directly 
--  to the connectors and are not connected to the CPLD.
--
--  System Bus Inputs:
--      db      - data bus (from system board)
--      len     - latch enable/gate signals (from system board)
--                  these connect to adr lines on system bus.
--                  LEN(0) corresponds to ADR(1) in the system bus.
--		cs		- board select input (from system board)
--
--  Local Inputs:
--      btn     - button inputs (on local board)
--
--  Local Outputs:
--      led     - 16 discrete leds
--      ssgan   - 4 seven segment display anode controls
--      ssgca   - 8 seven segment display cathode controls
--
------------------------------------------------------------------------
-- Revision History:
--  12/04/2002(GeneA): created
--  01/03/2003(GeneA): changed latch enables to be active low
--  02/19/2003(GeneA): Route LCD display panel data bus through CPLD.
--      This is needed due to the fact that the LCD is a 5V part, and
--      some FPGA devices don't have 5V tolerant i/o's. The CPLD
--      is a 3V part with 5V tolerant i/os. This will convert 5V to 3V 
--      logic levels.
------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity DlabDio5 is
    Port ( db       : inout std_logic_vector(7 downto 0);
           cs       : in std_logic;
           we       : in std_logic;
           oe       : in std_logic;
           lcen     : in std_logic;
           adr0     : in std_logic;
           adr1     : in std_logic;
           adr2     : in std_logic;
           adr3     : in std_logic;
           adr4     : in std_logic;
           adr5     : in std_logic;
           rgbLcd   : inout std_logic_vector(7 downto 0);
           lcden    : out std_logic;
           lcdrw    : out std_logic;
           lcdrs    : out std_logic;
           rgbBtn   : in std_logic_vector(15 downto 0);
           encBtn   : out std_logic_vector(3 downto 0);
           rgbLed   : out std_logic_vector(15 downto 0);
           rgbSsgAn : out std_logic_vector(3 downto 0);
           rgbSsgCa : out std_logic_vector(7 downto 0);
           kbcs     : out std_logic;
           kbds     : out std_logic;
           kbck     : in std_logic;
           kbdk     : in std_logic
    );
end DlabDio5;

architecture Behavioral of DlabDio5 is


	------------------------------------------------------------------------
	-- Component Declarations
	------------------------------------------------------------------------


	------------------------------------------------------------------------
	-- Signal Declarations
	------------------------------------------------------------------------

    signal  busDin          : std_logic_vector(7 downto 0);
    signal  busDout         : std_logic_vector(7 downto 0);

    -- Internal registers for output devices
    signal  regLedLo        : std_logic_vector(7 downto 0);
    signal  regLedHi        : std_logic_vector(7 downto 0);
    signal  regSsgCa        : std_logic_vector(7 downto 0);
    signal  regSsgAn        : std_logic_vector(3 downto 0);

    -- Write strobes for updating output device registers
    signal  lenLedLo        : std_logic;
    signal  lenLedHi        : std_logic;
    signal  lenSsgCa        : std_logic;
    signal  lenSsgAn        : std_logic;

    signal  selLcd          : std_logic;
    signal  selBtnHi        : std_logic;
    signal  selBtnLo        : std_logic;

----------------------------------------------------------------------------
--                      Module Implementation
----------------------------------------------------------------------------
    
begin

    ------------------------------------------------------------------------
    -- General Combinational Logic
    ------------------------------------------------------------------------

    db <= busDout when oe = '0' else "ZZZZZZZZ";
    busDin <= db;
    busDout <= rgbLcd when selLcd = '1' else
               rgbBtn(15 downto 8) when selBtnHi = '1' else
               rgbBtn( 7 downto 0);

    -- Assign outputs of the LED holding registers to the led outputs
    rgbLed(15 downto 8)  <= regLedHi;
    rgbLed( 7 downto 0)  <= regLedLo;

    -- Assign seven segment display outputs from the holding registers
    rgbSsgCa <= regSsgCa;
    rgbSsgAn <= regSsgAn;

    selBtnHi <= '1' when cs = '0' and adr5 = '1' and adr0 = '1' else '0';
    selBtnLo <= '1' when cs = '0' and adr5 = '1' and adr0 = '0' else '0';

	------------------------------------------------------------------------
    -- Synthesize control signals for the LCD display
	------------------------------------------------------------------------

    rgbLcd <= db when (cs = '0' and we = '0') else "ZZZZZZZZ";

    selLcd <= '1' when cs = '0' and adr5 = '0' else '0';
    lcden  <= '1' when selLcd = '1' and lcen = '1' else '0';
--    lcden  <= '1' when selLcd = '1' and ((we = '0') or (oe = '0')) else '0';
    lcdrw  <= we;
    lcdrs  <= adr0;

	------------------------------------------------------------------------
    -- Keyboard Interface Pass Through
	------------------------------------------------------------------------

    kbcs <= kbck;
    kbds <= kbdk;

	------------------------------------------------------------------------
    -- Latches for On-Board Output Devices
	------------------------------------------------------------------------

    -- Generate latch enables for the data registers.
    lenLedLo  <= '1' when cs = '0' and adr1 = '0' else '0';
    lenLedHi  <= '1' when cs = '0' and adr2 = '0' else '0';
    lenSsgCa  <= '1' when cs = '0' and adr3 = '0' else '0';
    lenSsgAn  <= '1' when cs = '0' and adr4 = '0' else '0';

    -- The discrete led and seven segment display signals are latched from
    -- the data bus via transparent latches. If the system board sets
    -- CS='0' and adr1='0' (for example) then the low byte of the leds
    -- will track the data on the data bus and give a transparent output
    -- on those leds.

    -- Latch low byte of leds
    process (lenLedLo, busDin)
        begin
            if lenLedLo = '1' then
                regLedLo <= busDin;
            end if;
        end process;

    -- Latch high byte of leds
    process (lenLedHi, busDin)
        begin
            if lenLedHi = '1' then
                regLedHi <= busDin;
            end if;
        end process;

    -- Latch the seven segment display cathode signals
    process (lenSsgCa, busDin)
        begin
            if lenSsgCa = '1' then
                regSsgCa <= busDin;
            end if;
        end process;

    -- Latch the seven segment display anode signals
    process (lenSsgAn, busDin)
        begin
            if lenSsgAn = '1' then
                regSsgAn <= busDin(3 downto 0);
            end if;
        end process;

	------------------------------------------------------------------------
    -- 10-4 Line BCD Encoder for Buttons
	------------------------------------------------------------------------

    encBtn <=   "0000" when rgbBtn(0) = '1' else
                "0001" when rgbBtn(1) = '1' else
                "0010" when rgbBtn(2) = '1' else
                "0011" when rgbBtn(3) = '1' else
                "0100" when rgbBtn(4) = '1' else
                "0101" when rgbBtn(5) = '1' else
                "0110" when rgbBtn(6) = '1' else
                "0111" when rgbBtn(7) = '1' else
                "1000" when rgbBtn(8) = '1' else
                "1001" when rgbBtn(9) = '1' else
                "1111";

	------------------------------------------------------------------------

end Behavioral;
